/*
 *  Name: prg2msg.c
 *  Author: Vincent Lefvre
 *  Date: 17 / 11 / 1994
 *
 *  .PRG --> .MSG converter
 *
 *  This converter  is  freeware.  It is  a utility  for the shareware game
 *  TETRHEX (N.B.: TETRHEX is NOT freeware).
 *
 *  This program  converts  a runnable .PRG file  containing a message file
 *  routine  into a message file.  The runnable file  should be composed of
 *  the header,  'MSG1' (the first long word in the TEXT segment), then the
 *  routine and data.  The DATA segment and the BSS segment should be empty
 *  (data should be stored  in the TEXT segment  and  the routine  can do a
 *  Malloc for non-initialized variables). There should be no symbol table.
 *  After the TEXT segment,  there should be a 0 long word, or no data (the
 *  routine should be relocatable).
 *
 *  The file name, with an extension or not,  should be given as the unique
 *  argument; if there's no extension, "PRG" is added as the extension. The
 *  result is stored in a file having the same name, but whose extension is
 *  "MSG".  The file name can begin with the path  (i.e. one has the path +
 *  the file name);  the length of the string shouldn't be greater than 255
 *  characters.
 *
 *  Error codes:
 *    0: No error.
 *    1: Usage (1 parameter).
 *    2: String too long (255 characters max).
 *    3: Cannot open the source file (the file doesn't exist?).
 *    4: Bad source file (see above).
 *    5: Insufficient memory.
 *    6: Cannot close the source file (shouldn't happen).
 *    7: Cannot create the target file.
 *    8: Cannot write in the target file (full disk?).
 *    9: Cannot close the target file (shouldn't happen).
 */

/*
 *  Nom: prg2msg.c
 *  Auteur: Vincent Lefvre
 *  Date: 17 / 11 / 1994
 *
 *  Convertisseur .PRG --> .MSG
 *
 *  Ce programme  est freeware.  C'est un utilitaire  pour le jeu shareware
 *  TETRHEX (N.B.: TETRHEX n'est PAS freeware).
 *
 *  Ce programme permet de convertir un excutable contenant une routine de
 *  fichier de messages en fichier de messages.  L'excutable doit contenir
 *  le header,  'MSG1' (premier mot long du segment TEXT),  ensuite on doit
 *  avoir la routine et les donnes. Il ne doit pas y avoir de segment DATA
 *  (les donnes doivent tre places dans le segment TEXT),  ni de segment
 *  BSS  (la routine doit faire un Malloc au moment de l'initialisation  si
 *  elle a besoin de donnes non initialises).  Il ne doit pas  y avoir de
 *  table des symboles  non plus.  Enfin, il doit y avoir  un mot long nul,
 *  ou aucune autre donne (car la routine doit tre relogeable).
 *
 *  On doit donner comme unique paramtre  le nom du fichier,  avec ou sans
 *  extension;  s'il n'y a pas d'extension,  l'extension "PRG" est ajoute.
 *  Le rsultat  est stock  dans un fichier  ayant le mme nom,  mais dont
 *  l'extension est "MSG". Le nom du fichier donn en paramtre peut conte-
 *  nir le chemin;  la longueur totale de la chane ne doit pas excder 255
 *  caractres.
 *
 *  Codes d'erreur:
 *    0: pas d'erreur.
 *    1: Usage (1 paramtre).
 *    2: Chane trop longue (255 caractres max).
 *    3: Le fichier source ne peut pas tre ouvert (il n'existe pas?).
 *    4: Mauvais fichier source (voir ci-dessus).
 *    5: Mmoire insuffisante.
 *    6: Le fichier source ne peut pas tre ferm (ne devrait pas arriver).
 *    7: Le fichier cible ne peut pas tre cr.
 *    8: Impossible d'crire dans le fichier cible (disque plein?).
 *    9: Le fichier cible ne peut pas tre ferm (ne devrait pas arriver).
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLEN 256

main(int argc, char *argv[])
{
  long n[4];  /* long int: 32 bits */
  char buffer[MAXLEN], *s;
  FILE *f;

  if (argc != 2)
    { fprintf(stderr, "Usage: prg2msg <file>\n");
      return 1; }

  if (strlen(argv[1]) > MAXLEN-5)
    { fprintf(stderr, "Error: string too long!\n");
      return 2; }

  if (!(s = strrchr(strcpy(buffer, argv[1]), '\\')))  s = buffer;

  while (*(s++) != '.')
    if (!*s)
      { strcpy(s++, ".PRG");
        break; }

  if (!(f = fopen(buffer, "rb")))
    { fprintf(stderr, "Cannot open file %s!\n", buffer);
      return 3; }

  strcpy(s, "MSG");

  if (fgetc(f) != 0x60 || fgetc(f) != 0x1A || !fread(n, 16, 1, f)
      || n[1] || n[2] || n[3] || fseek(f, 10, SEEK_CUR))
    { bad_source_file:
      fprintf(stderr, "Bad source file!\n");
      return 4; }

  if (!(s = (char *) malloc(n[0])))
    { fprintf(stderr, "Insufficient memory!\n");
      return 5; }

  if (!fread(s, n[0], 1, f) || (!feof(f) && (!fread(n+1, 4, 1, f) || n[1])))
    goto bad_source_file;

  if (fclose(f))
    { fprintf(stderr, "Cannot close the source file!\n");
      return 6; }

  if (!(f = fopen(buffer, "wb")))
    { fprintf(stderr, "Cannot create the file %s!\n", buffer);
      return 7; }

  if (!fwrite(s, n[0], 1, f))
    { fprintf(stderr, "Cannot write in the target file!\n");
      return 8; }

  if (fclose(f))
    { fprintf(stderr, "Cannot close the target file!\n");
      return 9; }

  return 0;
}
